@@ -31,6 +31,7 @@ static void             gail_toplevel_object_finalize   (GObject
 static gint             gail_toplevel_get_n_children    (AtkObject              *obj);
 static AtkObject*       gail_toplevel_ref_child         (AtkObject              *obj,
                                                         gint                    i);
+static AtkObject*       gail_toplevel_get_parent        (AtkObject              *obj);
 
 /* Callbacks */
 
@@ -90,12 +91,13 @@ gail_toplevel_new (void)
   GObject *object;
   AtkObject *accessible;
 
-  object = g_object_new(GAIL_TYPE_TOPLEVEL, NULL);
+  object = g_object_new (GAIL_TYPE_TOPLEVEL, NULL);
   g_return_val_if_fail ((object != NULL), NULL);
 
   accessible = ATK_OBJECT (object);
   accessible->role = ATK_ROLE_APPLICATION;
   accessible->name = g_get_prgname();
+  accessible->accessible_parent = NULL;
 
   return accessible;
 }
@@ -110,6 +112,7 @@ gail_toplevel_class_init (GailToplevelClass *klass)
 
   class->get_n_children = gail_toplevel_get_n_children;
   class->ref_child = gail_toplevel_ref_child;
+  class->get_parent = gail_toplevel_get_parent;
 
   g_object_class->finalize = gail_toplevel_object_finalize;
 }
@@ -171,6 +174,12 @@ gail_toplevel_object_finalize (GObject *obj)
   G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
+static AtkObject*
+gail_toplevel_get_parent (AtkObject *obj)
+{
+    return NULL;
+}
+
 static gint
 gail_toplevel_get_n_children (AtkObject *obj)
 {
@@ -239,20 +248,37 @@ gail_toplevel_show_event_watcher (GSignalInvocationHint *ihint,
       GTK_IS_PLUG (widget))
     return TRUE;
 
-  /* Add the window to the list & emit the signal */
-  toplevel->window_list = g_list_append (toplevel->window_list, widget);
-
-  n_children = g_list_length (toplevel->window_list);
-
- /*
-  * Must subtract 1 from the n_children since the index is 0-based
-  * but g_list_length is 1-based.
-  */
   child = gtk_widget_get_accessible (widget);
-  atk_object_set_parent (child, atk_obj);
-  g_signal_emit_by_name (atk_obj, "children-changed::add",
-                         n_children - 1, 
-                         child, NULL);
+  if (!strcmp (atk_role_get_name (atk_object_get_role (child)), "redundant object"))
+    {
+      return TRUE;
+    }
+
+  child = gtk_widget_get_accessible (widget);
+  if (!strcmp (atk_role_get_name (atk_object_get_role (child)), "redundant object"))
+    {
+      return TRUE;
+    }
+
+  /* 
+   * Add the window to the list & emit the signal.
+   * Don't do this for tooltips (Bug #150649).
+   */
+  if (atk_object_get_role (child) != ATK_ROLE_TOOL_TIP)
+  {
+      toplevel->window_list = g_list_append (toplevel->window_list, widget);
+
+      n_children = g_list_length (toplevel->window_list);
+
+      /*
+       * Must subtract 1 from the n_children since the index is 0-based
+       * but g_list_length is 1-based.
+       */
+      atk_object_set_parent (child, atk_obj);
+      g_signal_emit_by_name (atk_obj, "children-changed::add",
+			     n_children - 1, 
+			     child, NULL);
+  }
 
   /* Connect destroy signal callback */
   g_signal_connect (G_OBJECT(object), 
